import { bundleMDX } from 'mdx-bundler';
import remarkMdxImages from 'remark-mdx-images';
import rehypeSlug from 'rehype-slug';
import gfm from 'remark-gfm';
import rehypeAutolinkHeadings from 'rehype-autolink-headings';
import { extract_toc } from './extract_toc.js';

class MarkdownTransformer {
    log;
    workdir;
    outdir;
    constructor(log, workdir, outdir) {
        this.log = log;
        this.workdir = workdir;
        this.outdir = outdir;
    }
    async bundle_markdown({ bundle_name, source, cwd }) {
        const toc = await extract_toc(source);
        const { code, frontmatter } = await bundleMDX({
            source,
            cwd,
            bundleDirectory: this.outdir,
            bundlePath: "/file/",
            mdxOptions: (options) => {
                options.remarkPlugins = [
                    ...(options.remarkPlugins ?? []),
                    remarkMdxImages,
                    gfm,
                ];
                options.rehypePlugins = [
                    ...(options.rehypePlugins ?? []),
                    rehypeSlug,
                    [rehypeAutolinkHeadings, { behavior: "wrap" }],
                ];
                return options;
            },
            esbuildOptions: (options) => {
                // Set the `outdir` to a public location for this bundle.
                options.outdir = this.outdir;
                options.loader = {
                    ...options.loader,
                    // Tell esbuild to use the `file` loader for pngs
                    ".png": "file",
                    ".webp": "file",
                    ".jpg": "file",
                    ".gif": "file",
                    ".mp4": "file",
                };
                // Set the public path to /img/about
                //options.publicPath = `${box_name}/${bundle_name}`;
                // NOTE: @boyander, remove box name
                options.publicPath = `${bundle_name}`;
                // Set write to true so that esbuild will output the files.
                options.write = true;
                return options;
            },
        });
        return {
            code,
            frontmatter: {
                ...frontmatter,
                modeldata: {
                    toc: JSON.stringify(toc),
                    ...frontmatter.modeldata,
                },
            },
        };
    }
}

export { MarkdownTransformer };
