import { globby } from 'globby';
import fs from 'fs-extra';
import { list_all_in_box, upsert_post } from '../../api/api.js';
import PQueue from 'p-queue';
import * as R from 'ramda';
import { createRequire } from 'module';

const require = createRequire(import.meta.url);
const RateLimiter = require("limiter").RateLimiter;
const rateLimiter = new RateLimiter({
    tokensPerInterval: 2,
    interval: 300,
});
const queue = new PQueue({ concurrency: 5 }); // set a max concurrency of 5 requests
const uploadFileData = async ({ source_dir, filename, box, slug, }) => {
    const content = await fs.readJSON(`${source_dir}/${filename}`);
    await upsert_post(slug, content, box);
};
var handler = async ({ srcdir, box }) => {
    // Scan source dir for makrdown files
    const files = await globby("**/*.json", {
        cwd: srcdir,
        ignore: ["**/node_modules"],
    });
    const slugs = files.map((filename) => ({
        slug: filename.replace(".json", ""),
        filename,
    }));
    // Diff already uploaded slugs
    const posts = await list_all_in_box({ box });
    const diff_slugs = R.difference(posts.map((p) => p.slug), slugs.map((e) => e.slug));
    // TODO: delete diff slugs
    console.log("Slugs to be deleted");
    console.log(diff_slugs);
    // Upload new data
    for (let { slug, filename } of slugs) {
        await rateLimiter.removeTokens(1);
        await queue.add(() => uploadFileData({ slug, source_dir: srcdir, box, filename }));
    }
};

export { handler as default };
